--Use this query to find the Top 10 queries, aggregated over different intevals, for a period of time.
--The output of this query can be fed into a BI/Reporting tool for a visual analysis of query performance
--Creator: Anat Dror, Quest

declare
@history_end_time datetimeoffset(7),
@history_start_time datetimeoffset(7),

@results_row_count int = 1000;

set @history_end_time = GETUTCDATE();
set @history_start_time = DATEADD(day, -2, GETUTCDATE());  


with stats as(
    SELECT 
        p.query_id query_id,
		rs.runtime_stats_interval_id,
        CONVERT(float, SUM(rs.avg_cpu_time*rs.count_executions)) total_cpu_time,
        CONVERT(float, SUM(rs.avg_duration*rs.count_executions)) total_duration,
        CONVERT(float, SUM(rs.avg_logical_io_writes*rs.count_executions)) total_logical_io_writes,
        CONVERT(float, SUM(rs.avg_logical_io_reads*rs.count_executions)) total_logical_io_reads,
        CONVERT(float, SUM(rs.avg_query_max_used_memory*rs.count_executions)) total_query_max_used_memory,
        CONVERT(float, SUM(rs.avg_physical_io_reads*rs.count_executions)) total_physical_io_reads,
        SUM(rs.count_executions) count_executions,
		AVG(rs.avg_duration) avg_duration,
		row_number() over (partition by runtime_stats_interval_id order by SUM(rs.avg_duration*rs.count_executions) desc) as row#
    FROM sys.query_store_runtime_stats rs
        JOIN sys.query_store_plan p ON p.plan_id = rs.plan_id
    WHERE NOT (rs.first_execution_time > @history_end_time OR rs.last_execution_time < @history_start_time)
	and execution_type = 0
	
    GROUP BY rs.runtime_stats_interval_id,p.query_id
	)
	select *
	from stats
	where row# <= 10
	