--Script to demonstrate the SQL Server 2016 Query Store
--This example creates a table and a stored procedure, that when run will use two different plans
--After it is complete, the Query Store reports will show execution performance.
--Creator: Anat Dror, Quest


Use PASS_QueryStoreExample
GO
-- introduce table 

DROP TABLE IF EXISTS Quotations
GO
CREATE TABLE Quotations
(
	ID INT NOT NULL IDENTITY PRIMARY KEY CLUSTERED,
	Name VARCHAR(100) NOT NULL,	
	Quote VARCHAR(5000) NOT NULL,
	Extra VARCHAR(100),
	Iteration INT  NOT NULL
)
GO
 
-- Create a supporting new Non-Clustered Index.
CREATE NONCLUSTERED INDEX idx_Iteration ON Quotations(Iteration)
GO
 
 
-- Insert 90000 records
DECLARE @i INT = 1
WHILE (@i <= 30000)
BEGIN
	INSERT INTO Quotations(Quote,Name,Extra,Iteration) 
	VALUES  ('Life can only be understood backwards, but it must be lived forwards.','Soren Kierkegaard','(1813 - 1855)',@i),
          ('There is never enough time, unless you are serving it.','Malcolm Forbes','(1919 - 1990)',@i),
        	                ('It is nobler to declare oneself wrong than to insist on being right - especially when one is right.','Friedrich Nietzsche','(1844 - 1900) Thus Spoke Zarathustra',@i),
        	                ('Man is tormented by no greater anxiety than to find someone quickly to whom he can hand over that great gift of freedom with which the ill-fated creature is born.','Fyodor Dostoevsky','(1821 - 1881), The Brothers Karamazov',@i),
        	                ('I am all in favor of keeping dangerous weapons out of the hands of fools. Let us start with typewriters.','Solomon Short',NULL,@i),
        	                ('Let us make a special effort to stop communicating with each other, so we can have some conversation.','Judith Martin','(Miss Manners)',@i),
        	                ('No animal should ever jump up on the dining-room furniture unless absolutely certain that he can hold his own in the conversation.','Fran Lebowitz','(1950 - )',@i),
        	                ('The most radical revolutionary will become a conservative the day after the revolution.','Hannah Arendt','(1906 - 1975)',@i),
        	                ('Curiosity killed the cat, but for a while I was a suspect.','Steven Wright','(1955 - )',@i),
        	                ('For the skeptic there remains only one consolation: if there should be such a thing as superhuman law it is administered with subhuman inefficiency.','Eric Ambler',NULL,@i),
        	                ('It is a sign of a creeping inner death when we no longer can praise the living.','Eric Hoffer','(1902 - 1983)',@i),
        	                ('I have seen the future and it does not work.','Robert Fulford',NULL,@i),
        	                ('Nothing fixes a thing so intensely in the memory as the wish to forget it.','Michel de Montaigne','(1533 - 1592)',@i),
                        	('Punctuality is the virtue of the bored.','Evelyn Waugh','(1903 - 1966), Diaries of Evelyn Waugh (1976)',@i),
                            ('I quit therapy because my analyst was trying to help me behind my back.','Richard Lewis',NULL,@i),
                        	('Those who speak most of progress measure it by quantity and not by quality.','George Santayana','(1863 - 1952)',@i),
                        	('A Hospital is no place to be sick.','Samuel Goldwyn','(1882 - 1974)',@i),
                        	('Eternal nothingness is fine if you happen to be dressed for it.','Woody Allen','(1935 - )',@i),
                        	('It is impossible to defeat an ignorant man in argument.','William G. McAdoo','(1863 - 1941)',@i),
                        	('Television is more interesting than people. If it were not, we would have people standing in the corners of our rooms.','Unknown',NULL,@i),
                        	('I am a Marxist--of the Groucho tendency.','Anonymous','French slogan',@i),
                            ('The first principle is that you must not fool yourself - and you are the easiest person to fool.','Richard Feynman','(1918 - 1988)',@i),
                        	('Do not tell people how to do things. Tell them what to do and let them surprise you with their results.','George S. Patton','(1885 - 1945)',@i),
                        	('No one gossips about other peoples secret virtues.','Bertrand Russell','(1872 - 1970)',@i),
                        	('The young have aspirations that never come to pass, the old have reminiscences of what never happened.','Saki','(1870 - 1916)',@i),
                        	('It is well to remember that the entire universe, with one trifling exception, is composed of others.','John Andrew Holmes',NULL,@i),
                        	('A coupla months in the laboratory can save a coupla hours in the library.','Westheimers Discovery',NULL,@i),
                        	('A door is what a dog is perpetually on the wrong side of.','Ogden Nash','(1902 - 1971)',@i),
                        	('Getting caught is the mother of invention.','Robert Byrne',NULL,@i),
                        	('Good judgment comes from experience, and experience comes from bad judgment.','Barry LePatner',NULL,@i),
                        	('Nothing is more conducive to peace of mind than not having any opinions at all.','Georg Christoph Lichtenberg','(1742 - 1799)',@i),
                            ('It is a mistake to think you can solve any major problems just with potatoes.','Douglas Adams','(1952 - 2001)',@i );	
	SET @i += 1
END
GO



-- show size of table

			select 
			max(t.object_id) row_id,
			'PASS_QueryStoreExample' db_name,
			s.name COLLATE SQL_Latin1_General_CP1_CI_AS owner_name,
			t.name COLLATE SQL_Latin1_General_CP1_CI_AS table_name,
			'PASS_QueryStoreExample' + '.' + IsNull ( s.name COLLATE SQL_Latin1_General_CP1_CI_AS , '<unknown-user>' ) + '.' + t.name COLLATE SQL_Latin1_General_CP1_CI_AS As qualified_name,
			isnull(max(fg.name),'Not applicable') COLLATE SQL_Latin1_General_CP1_CI_AS fg_name,
			SUM (CASE	WHEN au.type <> 1 THEN au.used_pages 
						WHEN p.index_id < 2	THEN au.data_pages 
						ELSE 0 
						END) * 8 / 1024.0 table_mb,
			max(si.reserved) reserved,
			max(si.dpages) used,
			max(si.reserved-si.dpages) free,			 
			100.0*(max(si.reserved)*8.0/1024)/(select sum(size)*8.0/1024 from [PASS_QueryStoreExample].sys.database_files where type !=1) percent_of_db,
			max(si.Rows) rows
			from [PASS_QueryStoreExample].sys.tables t (nolock)
			left outer join [PASS_QueryStoreExample].sys.schemas s (nolock)
				on t.schema_id=s.schema_id
			left outer join [PASS_QueryStoreExample].sys.indexes i (nolock)
				on i.object_id=t.object_id
				and i.index_id<2
			left outer join [PASS_QueryStoreExample].sys.sysindexes si (nolock)
				on	si.id=t.object_id 
				and si.indid=i.index_id					
			left outer join [PASS_QueryStoreExample].sys.partitions p (nolock) 
				on p.object_id=t.object_id 
			left outer join [PASS_QueryStoreExample].sys.allocation_units As au (NOLOCK) 
				ON au.container_id = p.hobt_id
				And	au.type In ( 1 , 3 ) 
				Or au.container_id = p.partition_id
						And		au.type = 2 
			left outer join [PASS_QueryStoreExample].sys.filegroups fg on au.data_space_id = fg.data_space_id
		  where t.name='Quotations'  
		  
		  group by 
		  s.name COLLATE SQL_Latin1_General_CP1_CI_AS,
		  t.name COLLATE SQL_Latin1_General_CP1_CI_AS
		   


-- about 17260 pages allocated and 15000 are used

-- enable QueryStore
ALTER DATABASE PASS_QueryStoreExample
SET QUERY_STORE = ON
GO
 
-- Configure the Query Store
ALTER DATABASE PASS_QueryStoreExample SET QUERY_STORE
(
OPERATION_MODE = READ_WRITE, 
CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 30), --number of days to retain data in the query store
DATA_FLUSH_INTERVAL_SECONDS = 900, 		    --frequency at which data is persisted to disk
INTERVAL_LENGTH_MINUTES = 60,                       --interval to aggregate runtime execution statistics MAX_STORAGE_SIZE_MB = 1024, 		    --maximum size of the query store
QUERY_CAPTURE_MODE = ALL,        		    --type of queries Query Store captures 
				                            --(all, relevant, tracked)
MAX_PLANS_PER_QUERY=5,                              --maximum number of plans maintained for each query. 
SIZE_BASED_CLEANUP_MODE = AUTO 		    --Controls the cleanup process
)
GO

-- create procedure to retrieve transactions based on range of CA_ID and 
DROP Procedure if exists RetrieveQuotes
go
CREATE PROCEDURE RetrieveQuotes
(
	@Value INT
)
AS
BEGIN
	SELECT * FROM Quotations
	WHERE Iteration <  @Value 
END
GO
-- run the procedure using parameters that will ensure high number of rows be returned
-- enable live query statistics
set statistics io on
go
exec RetrieveQuotes 30000
-- it takes about a minute to finish and use clustered index scan becuase the query is not selective enough 
-- to use and index seek. key lookups will take too long
-- now simulate restart
DBCC FREEPROCCACHE
GO
exec RetrieveQuotes 2
-- read 163 pages using index
-- now we go back to use 30000 again
exec RetrieveQuotes 30000
-- now instead of 15000 pages, there are almost 3M logical reads!


