--Use these queries to look at Query Store collected data for specific use cases
--Creator: Anat Dror, Quest

--example 1: find all queries that aborted during specific date
select  qt.query_sql_text,  q.query_hash, 
		q.query_id, qt.query_text_id, p.plan_id,sum(count_executions) as executions#
from
sys.query_store_query_text AS qt   
JOIN sys.query_store_query AS q   
    ON qt.query_text_id = q.query_text_id   
JOIN sys.query_store_plan AS p   
    ON q.query_id = p.query_id   
JOIN sys.query_store_runtime_stats AS rs   
    ON p.plan_id = rs.plan_id   
JOIN sys.query_store_runtime_stats_interval si
    ON si.runtime_stats_interval_id=rs.runtime_stats_interval_id
where execution_type <> 0                           --only aborted executions
and rs.last_execution_time >= DATEADD(DAY, -14, GETUTCDATE())
group by qt.query_sql_text,   q.query_hash,
    q.query_id, qt.query_text_id, p.plan_id



-- example 2: queries require paramaterization - same query hash for multiple queries with the same plan
select qs.query_hash,count_queries,count_plans
from
(select query_hash,count(q.query_id) as count_queries
from sys.query_store_query q join
 sys.query_store_plan p
on q.query_id = p.query_id
group by query_hash) qs
join
(select query_hash,count(query_plan_hash) as count_plans 
 from
	( select distinct query_hash,query_plan_hash
	  from sys.query_store_query q
	  join sys.query_store_plan p
	  on q.query_id = p.query_id
	) as qp1
	group by query_hash
	) qp
on qs.query_hash=qp.query_hash
where count_queries > 10 and count_plans < 10



---- find an example of such query
select query_sql_text,query_plan_hash
from
sys.query_store_query q join
 sys.query_store_plan p
on q.query_id = p.query_id
join
sys.query_store_query_text t on q.query_text_id=t.query_text_id
where query_hash=0xF6F00C1429B6739D



--- example 3: find regressed queries by perccent of duration
declare
@history_end_time datetimeoffset(7),
@history_start_time datetimeoffset(7),
@recent_end_time datetimeoffset(7),
@recent_start_time datetimeoffset(7),
@results_row_count int = 1000;

set @history_end_time = GETUTCDATE();
set @history_start_time = DATEADD(day, -2, GETUTCDATE())  -- 2 days
set @recent_end_time = GETUTCDATE();
set @recent_start_time = DATEADD(MINUTE, -60, GETUTCDATE());  -- 1 hour

with hist AS
(
    SELECT
        p.query_id query_id,
        CONVERT(float, SUM(rs.avg_cpu_time*rs.count_executions)) total_cpu_time,
        CONVERT(float, SUM(rs.avg_duration*rs.count_executions)) total_duration,
        CONVERT(float, SUM(rs.avg_logical_io_writes*rs.count_executions)) total_logical_io_writes,
        CONVERT(float, SUM(rs.avg_logical_io_reads*rs.count_executions)) total_logical_io_reads,
        CONVERT(float, SUM(rs.avg_query_max_used_memory*rs.count_executions)) total_query_max_used_memory,
        CONVERT(float, SUM(rs.avg_physical_io_reads*rs.count_executions)) total_physical_io_reads,
        SUM(rs.count_executions) count_executions,
		AVG(rs.avg_duration) avg_duration,
        COUNT(distinct p.plan_id) num_plans
    FROM sys.query_store_runtime_stats rs
        JOIN sys.query_store_plan p ON p.plan_id = rs.plan_id
    WHERE NOT (rs.first_execution_time > @history_end_time OR rs.last_execution_time < @history_start_time)
	and execution_type = 0
    GROUP BY p.query_id
),
recent AS
(
    SELECT
        p.query_id query_id,
        CONVERT(float, SUM(rs.avg_duration*rs.count_executions)) total_duration,
        SUM(rs.count_executions) count_executions,
		AVG(rs.avg_duration) avg_duration,
        COUNT(distinct p.plan_id) num_plans
    FROM sys.query_store_runtime_stats rs
        JOIN sys.query_store_plan p ON p.plan_id = rs.plan_id
    WHERE NOT (rs.first_execution_time > @recent_end_time OR rs.last_execution_time < @recent_start_time)
	AND execution_type = 0
    GROUP BY p.query_id
)
SELECT TOP (@results_row_count) 
    results.query_id query_id,
    results.query_text query_text,
    results.additional_duration_workload additional_duration_workload,
	iif(results.total_duration_hist > 0,results.additional_duration_workload*100/results.total_duration_hist,0) duration_percent_change,
    results.total_duration_recent total_duration_recent,
    results.total_duration_hist total_duration_hist,
    ISNULL(results.count_executions_recent, 0) count_executions_recent,
    ISNULL(results.count_executions_hist, 0) count_executions_hist
FROM
(
    SELECT
        hist.query_id query_id,
        qt.query_sql_text query_text,
        ROUND(CONVERT(float, recent.total_duration/recent.count_executions-hist.total_duration/hist.count_executions)*(recent.count_executions), 2) additional_duration_workload,
        ROUND(recent.total_duration, 2) total_duration_recent, 
        ROUND(hist.total_duration, 2) total_duration_hist,
        recent.count_executions count_executions_recent,
        hist.count_executions count_executions_hist
    FROM hist 
        JOIN recent ON hist.query_id = recent.query_id        
        JOIN sys.query_store_query q ON q.query_id = hist.query_id
        JOIN sys.query_store_query_text qt ON q.query_text_id = qt.query_text_id
) AS results

ORDER BY  duration_percent_change DESC
OPTION (MERGE JOIN)
